module PrintingPress
  module Controller

    def self.included(base)
      base.prepend_before_filter :set_printing_press_enabled_for_controller, :setup_printing_press
      base.after_filter :disable_printing_press	
      base.send :extend, ClassMethods
    end

    module ClassMethods
			def acts_as_publication(models = [], options = {})
				models = [ models] unless models.is_a?(Array) 
				
				class_attribute :printing_press_models
				self.printing_press_models = models
			end
		end
		
		protected
		
		def should_acts_as_publication?
      true
    end
		
		private
		
		def set_printing_press_enabled_for_controller
			::PrintingPress.enabled_for_controller = should_acts_as_publication?
		end
		
		def setup_printing_press
			if ::PrintingPress.enabled_for_controller?
				(self.class.try(:printing_press_models) || []).each do |model_name|
					model_name.to_s.classify.constantize.try(:printing_press_on)
				end
			end
		end
		
		def disable_printing_press
			if ::PrintingPress.enabled_for_controller?
				(self.class.try(:printing_press_models) || []).each do |model_name|
					model_name.to_s.classify.constantize.try(:printing_press_off)
				end
			end
		end
  end
end